package com.nschairer.keyboard_height_plugin
import android.graphics.Rect
import android.util.Log
import android.view.View
import androidx.annotation.NonNull
import android.view.ViewTreeObserver
import android.view.WindowInsetsController
import androidx.core.view.ViewCompat
import androidx.core.view.WindowInsetsCompat
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding

class KeyboardHeightPlugin : FlutterPlugin, EventChannel.StreamHandler, ActivityAware {
    private val keyboardHeightEventChannelName = "keyboardHeightEventChannel"
    private var eventSink: EventChannel.EventSink? = null
    private var eventChannel: EventChannel? = null
    private var activityPluginBinding: ActivityPluginBinding? = null
    private var bottomInset: Int = 0

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        eventChannel = EventChannel(flutterPluginBinding.binaryMessenger, keyboardHeightEventChannelName)
        eventChannel?.setStreamHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        eventChannel?.setStreamHandler(null)
    }

    fun handleKeyboardHeightCheck(rootView: View?) {
        rootView?.viewTreeObserver?.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                val r = Rect()
                rootView.getWindowVisibleDisplayFrame(r)

                val screenHeight = rootView.height
                // Also subtract the height of the bottom inset as the SafeArea with "bottom: false"
                // allows us to draw under the bottom system bar, if it is there.
                val keypadHeight = screenHeight - r.bottom - bottomInset

                val displayMetrics = activityPluginBinding?.activity?.resources?.displayMetrics
                val logicalKeypadHeight = keypadHeight / (displayMetrics?.density ?: 1f)

                if (keypadHeight > screenHeight * 0.15) {
                    eventSink?.success(logicalKeypadHeight.toDouble())
                } else {
                    eventSink?.success(0.0)
                }
            }
        })
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        eventSink = events
        val rootView = activityPluginBinding?.activity?.window?.decorView?.rootView
        handleKeyboardHeightCheck(rootView)

        if (rootView != null) {
            ViewCompat.setOnApplyWindowInsetsListener(rootView!!) { view, windowInsets ->
                val insets = windowInsets.getInsets(WindowInsetsCompat.Type.systemBars())
                val triggerEvent = bottomInset != insets.bottom
                bottomInset = insets.bottom

                // Notify in case the inset changed
                if (triggerEvent) handleKeyboardHeightCheck(rootView)

                Log.d("keyboard_height_plugin", "Got inset of " + bottomInset.toString())
                WindowInsetsCompat.CONSUMED
            }
        }
    }

    override fun onCancel(arguments: Any?) {
        eventSink = null
    }

    // Implement ActivityAware methods
    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityPluginBinding = binding
    }

    override fun onDetachedFromActivityForConfigChanges() {
        activityPluginBinding = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activityPluginBinding = binding
    }

    override fun onDetachedFromActivity() {
        activityPluginBinding = null
    }
}
